package com.landray.kmss.sys.attachment.borrow.service.spring;

import org.hibernate.query.NativeQuery;
import org.hibernate.Session;
import com.landray.kmss.sys.quartz.interfaces.SysQuartzJobContext;
import org.apache.commons.lang3.time.DateUtils;
import com.landray.kmss.sys.workflow.interfaces.Event_SysFlowFinish;
import java.util.Map;
import com.landray.kmss.sys.attachment.forms.AttachmentDetailsForm;
import com.landray.kmss.util.SpringBeanUtil;
import com.landray.kmss.sys.attachment.model.SysAttMain;
import com.landray.kmss.util.StringUtil;
import java.util.List;
import com.landray.kmss.sys.organization.model.SysOrgElement;
import java.util.ArrayList;
import com.landray.kmss.util.UserUtil;
import com.landray.kmss.common.actions.RequestContext;
import java.util.Date;
import com.landray.kmss.util.DateUtil;
import com.landray.kmss.sys.attachment.borrow.forms.SysAttBorrowForm;
import com.landray.kmss.sys.attachment.borrow.model.SysAttBorrow;
import com.landray.kmss.common.convertor.ConvertorContext;
import com.landray.kmss.common.model.IBaseModel;
import com.landray.kmss.common.forms.IExtendForm;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;
import com.landray.kmss.sys.attachment.service.ISysAttMainCoreInnerService;
import com.landray.kmss.sys.lbpmservice.support.service.ILbpmProcessCoreService;
import com.landray.kmss.sys.notify.interfaces.ISysNotifyMainCoreService;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;
import com.landray.kmss.sys.attachment.borrow.service.ISysAttBorrowService;
import com.landray.kmss.sys.metadata.interfaces.ExtendDataServiceImp;

public class SysAttBorrowServiceImp extends ExtendDataServiceImp implements ISysAttBorrowService, ApplicationListener<ApplicationEvent>
{
    private ISysNotifyMainCoreService sysNotifyMainCoreService;
    private ILbpmProcessCoreService lbpmProcessCoreService;
    private ISysAttMainCoreInnerService sysAttMainService;
    private static Logger logger;
    private final String UPDATE_DOING_SQL = "update sys_att_borrow set fd_status = '1' where fd_status = '0' and doc_status = '30' and (fd_borrow_effective_time <= ? and fd_borrow_expire_time > ? or fd_duration = 0)";
    private final String UPDATE_DONE_SQL = "update sys_att_borrow set fd_status = '2' where doc_status = '30' and (fd_status = '1' or fd_status = '0') and fd_borrow_expire_time <= ?";
    private final String UPDATE_CLOSE_SQL = "update sys_att_borrow set fd_status = '3' where fd_id in (:fdIds)";
    
    static {
        SysAttBorrowServiceImp.logger = LoggerFactory.getLogger(SysAttBorrowServiceImp.class);
    }
    
    public IBaseModel convertBizFormToModel(final IExtendForm form, IBaseModel model, final ConvertorContext context) throws Exception {
        model = super.convertBizFormToModel(form, model, context);
        if (model instanceof SysAttBorrow) {
            final SysAttBorrow sysAttBorrow = (SysAttBorrow)model;
            final SysAttBorrowForm sysAttBorrowForm = (SysAttBorrowForm)form;
            if ((sysAttBorrow.getDocStatus() == null || sysAttBorrow.getDocStatus().startsWith("1")) && sysAttBorrowForm.getDocStatus() != null && (sysAttBorrowForm.getDocStatus().startsWith("1") || sysAttBorrowForm.getDocStatus().startsWith("2"))) {
                sysAttBorrow.setDocStatus(sysAttBorrowForm.getDocStatus());
            }
            if (sysAttBorrow.getFdBorrowEffectiveTime() != null && sysAttBorrow.getFdDuration() > 0L) {
                final Date fdBorrowExpireTime = DateUtil.getNextDay(sysAttBorrow.getFdBorrowEffectiveTime(), sysAttBorrow.getFdDuration().intValue());
                sysAttBorrow.setFdBorrowExpireTime(fdBorrowExpireTime);
            }
        }
        return model;
    }
    
    public IBaseModel initBizModelSetting(final RequestContext requestContext) throws Exception {
        final SysAttBorrow sysAttBorrow = new SysAttBorrow();
        sysAttBorrow.setDocCreateTime(new Date());
        final String fdReadEnable = requestContext.getParameter("fdReadEnable");
        sysAttBorrow.setFdReadEnable(Boolean.valueOf("true".equals(fdReadEnable)));
        final String fdDownloadEnable = requestContext.getParameter("fdDownloadEnable");
        sysAttBorrow.setFdDownloadEnable(Boolean.valueOf("true".equals(fdDownloadEnable)));
        final String fdCopyEnable = requestContext.getParameter("fdCopyEnable");
        sysAttBorrow.setFdCopyEnable(Boolean.valueOf("true".equals(fdCopyEnable)));
        final String fdPrintEnable = requestContext.getParameter("fdPrintEnable");
        sysAttBorrow.setFdPrintEnable(Boolean.valueOf("true".equals(fdPrintEnable)));
        sysAttBorrow.setFdStatus("0");
        sysAttBorrow.setDocCreator(UserUtil.getUser());
        final List<SysOrgElement> fdBorrowers = new ArrayList<SysOrgElement>();
        fdBorrowers.add((SysOrgElement)UserUtil.getUser());
        sysAttBorrow.setFdBorrowers((List)fdBorrowers);
        final String fdAttId = requestContext.getParameter("fdAttId");
        if (StringUtil.isNotNull(fdAttId)) {
            final SysAttMain main = (SysAttMain)this.sysAttMainService.findByPrimaryKey(fdAttId);
            sysAttBorrow.setFdAttId(fdAttId);
            sysAttBorrow.setDocSubject(main.getFdFileName());
        }
        return (IBaseModel)sysAttBorrow;
    }
    
    public void initCoreServiceFormSetting(final IExtendForm form, final IBaseModel model, final RequestContext requestContext) throws Exception {
        this.getLbpmProcessCoreService().initFormDefaultSetting(form, "sysAttBorrow", "sysAttBorrow", requestContext);
    }
    
    public ISysNotifyMainCoreService getSysNotifyMainCoreService() {
        if (this.sysNotifyMainCoreService == null) {
            this.sysNotifyMainCoreService = (ISysNotifyMainCoreService)SpringBeanUtil.getBean("sysNotifyMainCoreService");
        }
        return this.sysNotifyMainCoreService;
    }
    
    public ILbpmProcessCoreService getLbpmProcessCoreService() {
        if (this.lbpmProcessCoreService == null) {
            this.lbpmProcessCoreService = (ILbpmProcessCoreService)SpringBeanUtil.getBean("lbpmProcessCoreService");
        }
        return this.lbpmProcessCoreService;
    }
    
    public IExtendForm convertModelToForm(final IExtendForm form, final IBaseModel model, final RequestContext requestContext) throws Exception {
        final SysAttBorrowForm sysAttBorrowForm = (SysAttBorrowForm)super.convertModelToForm(form, model, requestContext);
        final Map attForms = (Map)sysAttBorrowForm.getAttachmentForms();
        final SysAttMain main = (SysAttMain)this.sysAttMainService.findByPrimaryKey(sysAttBorrowForm.getFdAttId());
        final AttachmentDetailsForm attForm = attForms.get(main.getFdKey());
        attForm.setFdModelId(main.getFdModelId());
        attForm.setFdModelName(main.getFdModelName());
        attForm.setFdKey(main.getFdKey());
        requestContext.setAttribute("fdKey", (Object)main.getFdKey());
        if (!attForm.getAttachments().contains((Object)main)) {
            attForm.getAttachments().add((Object)main);
        }
        return (IExtendForm)sysAttBorrowForm;
    }
    
    public void setSysAttMainService(final ISysAttMainCoreInnerService sysAttMainService) {
        this.sysAttMainService = sysAttMainService;
    }
    
    public void onApplicationEvent(final ApplicationEvent event) {
        if (event == null) {
            return;
        }
        final Object obj = event.getSource();
        if (!(obj instanceof SysAttBorrow)) {
            return;
        }
        if (!(event instanceof Event_SysFlowFinish)) {
            return;
        }
        try {
            final SysAttBorrow borrow = (SysAttBorrow)obj;
            if (borrow.getFdBorrowEffectiveTime() == null) {
                borrow.setFdBorrowEffectiveTime(DateUtil.getDate(0));
            }
            final String fdStatus = this.getStatus(borrow);
            borrow.setFdStatus(fdStatus);
            if (borrow.getFdBorrowExpireTime() == null && borrow.getFdDuration() > 0L) {
                final Date fdBorrowExpireTime = DateUtil.getNextDay(borrow.getFdBorrowEffectiveTime(), borrow.getFdDuration().intValue());
                borrow.setFdBorrowExpireTime(fdBorrowExpireTime);
            }
            this.getBaseDao().update((IBaseModel)borrow);
            this.updateBorrowNum(borrow.getFdAttId());
        }
        catch (final Exception e) {
            SysAttBorrowServiceImp.logger.error("更新附件借阅状态报错：", e);
        }
    }
    
    private void updateBorrowNum(final String attId) throws Exception {
        final String hql = "update SysAttMain set fdBorrowCount = fdBorrowCount + 1 where fdId = :fdId";
        this.getBaseDao().getHibernateSession().createQuery(hql).setParameter("fdId", (Object)attId).executeUpdate();
    }
    
    private String getStatus(final SysAttBorrow borrow) {
        if (!"30".equals(borrow.getDocStatus())) {
            return "0";
        }
        final Long fdDuration = borrow.getFdDuration();
        final Date fdNow = DateUtil.getDate(0);
        final Long fdNowTime = fdNow.getTime();
        final Date fdEffectiveStart = borrow.getFdBorrowEffectiveTime();
        if (fdEffectiveStart.getTime() < fdNowTime) {
            return "0";
        }
        if (fdDuration == 0L) {
            return "1";
        }
        final Date fdEffectiveEnd = DateUtils.addDays(fdEffectiveStart, fdDuration.intValue());
        if (fdNowTime >= fdEffectiveEnd.getTime()) {
            return "2";
        }
        return "1";
    }
    
    public void updateBorrowFdStatus(final SysQuartzJobContext context) throws Exception {
        try {
            final Date now = DateUtil.getDate(0);
            final Session session = this.getBaseDao().getHibernateSession();
            final NativeQuery nativeQuery = session.createNativeQuery("update sys_att_borrow set fd_status = '1' where fd_status = '0' and doc_status = '30' and (fd_borrow_effective_time <= ? and fd_borrow_expire_time > ? or fd_duration = 0)");
            nativeQuery.addSynchronizedQuerySpace("update sys_att_borrow").setDate(0, now).setDate(1, now).executeUpdate();
            final NativeQuery nativeQuery2 = session.createNativeQuery("update sys_att_borrow set fd_status = '2' where doc_status = '30' and (fd_status = '1' or fd_status = '0') and fd_borrow_expire_time <= ?");
            nativeQuery2.addSynchronizedQuerySpace("update sys_att_borrow").setDate(0, now).executeUpdate();
        }
        catch (final Exception e) {
            context.logError("定时任务更新借阅状态报错：", (Throwable)e);
            throw e;
        }
    }
    
    public void updateCloseStatus(final String[] fdIds) throws Exception {
        final Session session = this.getBaseDao().getHibernateSession();
        session.createNativeQuery("update sys_att_borrow set fd_status = '3' where fd_id in (:fdIds)").setParameterList("fdIds", (Object[])fdIds).executeUpdate();
    }
}
